##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
"""Tests of combining ZopeVersionControl with the References product.

$Id$
"""
import unittest

from .common import common_setUp
from .common import common_tearDown


noPathReference = False
try:
    from Products.References.PathReference import PathReference
except ModuleNotFoundError:
    noPathReference = True


@unittest.skipIf(noPathReference, 'Products.References not available')
class ReferenceVCTests(unittest.TestCase):

    setUp = common_setUp
    tearDown = common_tearDown

    def testContainerVersioning(self):
        # Verify that containers and items are versioned independently,
        # except in the case of references.
        from OFS.DTMLDocument import addDTMLDocument

        repository = self.repository
        folder1 = self.app.folder1
        folder2 = folder1.folder2
        folder1.testattr = 'container_v1'
        folder2.testattr = 'item_v1'
        folder1._setOb("ref", PathReference("ref", folder1.folder2))
        folder2._true_id = "folder2"
        self.assertEqual(folder1.ref._true_id, "folder2")

        self.assertFalse(repository.isUnderVersionControl(folder1))
        repository.applyVersionControl(folder1)
        folder1 = self.app.folder1
        self.assertTrue(repository.isUnderVersionControl(folder1))
        self.assertFalse(repository.isUnderVersionControl(folder2))
        self.assertFalse(repository.isUnderVersionControl(folder1.ref))
        repository.applyVersionControl(folder2)
        folder2 = folder1.folder2
        self.assertTrue(repository.isUnderVersionControl(folder2))
        self.assertFalse(repository.isUnderVersionControl(folder2.document1))

        # Make the first version of folder1 and check it in.
        repository.checkoutResource(folder1)
        folder1 = self.app.folder1
        repository.checkinResource(folder1)
        folder1 = self.app.folder1
        folder2 = folder1.folder2
        info = repository.getVersionInfo(folder1)
        first_version = info.version_id

        # Change folder1 and check it in again
        repository.checkoutResource(folder1)
        folder1 = self.app.folder1
        folder1.testattr = 'container_v2'
        addDTMLDocument(folder1, 'document3', file='some more text')
        folder1.document3._true_id = "document3"
        repository.checkinResource(folder1)
        folder1 = self.app.folder1
        folder2 = folder1.folder2

        # Change ref to point to document3.
        folder1._delObject("ref")
        folder1._setOb("ref", PathReference("ref", folder1.document3))
        self.assertEqual(folder1.ref._true_id, "document3")

        # Change folder2
        repository.checkoutResource(folder2)
        folder2 = folder1.folder2
        folder2.testattr = 'item_v2'

        # Now revert folder1 and verify that folder2 was not reverted.
        # Also verify that ref now points back to folder2.
        repository.updateResource(folder1, first_version)
        folder1 = self.app.folder1
        folder2 = folder1.folder2
        self.assertEqual(folder1.testattr, 'container_v1')
        self.assertEqual(folder2.testattr, 'item_v2')
        self.assertEqual(folder1.ref._true_id, "folder2")

        # Verify that document3 remains an item of the reverted folder1.
        self.assertTrue(hasattr(folder1, 'document3'))
        self.assertEqual(str(folder1.document3), 'some more text')

        # Remove document3 and verify that it doesn't reappear upon revert.
        folder1._delObject('document3')
        repository.updateResource(folder1, '')
        folder1 = self.app.folder1
        self.assertEqual(folder1.testattr, 'container_v2')
        self.assertEqual(folder1.folder2.testattr, 'item_v2')
        self.assertFalse(hasattr(folder1, 'document3'))



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
"""Unit testing utilities."""
import transaction


def common_setUp(self):
    # Install a hack to make SimpleItem version aware, so that the
    # tests work. In normal development, you would mix in the
    # VersionSupport class on an as-needed basis.
    from AccessControl.SecurityManagement import newSecurityManager
    from OFS.Application import Application
    from OFS.DTMLDocument import addDTMLDocument
    from OFS.Folder import manage_addFolder
    from Testing.makerequest import makerequest

    import Products.ZopeVersionControl
    Products.ZopeVersionControl.install_hack()

    from io import StringIO

    from ZODB import DB
    from ZODB.DemoStorage import DemoStorage

    from Products.ZopeVersionControl.ZopeRepository import addRepository

    s = DemoStorage()
    self.connection = DB(s).open()
    try:
        r = self.connection.root()
        a = Application()
        r['Application'] = a
        self.root = a
        responseOut = self.responseOut = StringIO()
        self.app = makerequest(self.root, stdout=responseOut)
        self.app.acl_users.userFolderAddUser('UnitTester', '123', (), ())
        manage_addFolder(self.app, 'folder1')
        self.folder1 = getattr(self.app, 'folder1')
        manage_addFolder(self.folder1, 'folder2')
        self.folder2 = getattr(self.folder1, 'folder2')
        addDTMLDocument(self.folder2, 'document1', file='some text')
        self.document1 = getattr(self.folder2, 'document1')
        addDTMLDocument(self.folder2, 'document2', file='some text')
        self.document2 = getattr(self.folder2, 'document2')
        addDTMLDocument(self.folder2, 'document_nonversion', file='some?')
        self.document_nonversion = getattr(self.folder2,
                                           'document_nonversion')
        self.document_nonversion.__non_versionable__ = 1
        addRepository(self.folder1, 'repository')
        self.repository = getattr(self.folder1, 'repository')
        transaction.commit()
    except BaseException:
        self.connection.close()
        raise
    transaction.begin()
    user = self.app.acl_users.getUser('UnitTester')
    user = user.__of__(self.app.acl_users)
    newSecurityManager(None, user)


def common_tearDown(self):
    from AccessControl.SecurityManagement import noSecurityManager
    noSecurityManager()
    del self.folder1
    del self.folder2
    del self.document1
    del self.document2
    transaction.abort()
    self.app._p_jar.sync()
    self.connection.close()
    del self.app
    del self.responseOut
    del self.root
    del self.connection


def common_commit(self):
    if self.do_commits:
        transaction.commit()



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import AccessControl
import ExtensionClass
from AccessControl.class_init import InitializeClass
from App.special_dtml import DTMLFile

from .Utility import VersionControlError
from .Utility import isAVersionableResource
from .Utility import use_vc_permission


class VersionSupport(ExtensionClass.Base):
    """Mixin class to support version-controllable resources."""

    manage_options = (
        {'label': 'Version Control', 'action': 'versionControlMain',
         'help': ('ZopeVersionControl', 'VersionControl.stx'),
         'filter': isAVersionableResource,
         },
    )

    security = AccessControl.ClassSecurityInfo()

    security.declareProtected('View management screens', 'versionControlMain')
    versionControlMain = DTMLFile('dtml/VersionControlMain', globals())

    security.declareProtected('View management screens', 'versionControlLog')
    versionControlLog = DTMLFile('dtml/VersionControlLog', globals())

    @security.private
    def haveRepository(self):
        try:
            self.getRepository()
        except VersionControlError:
            return 0
        return 1

    @security.private
    def getRepository(self):
        # We currently only allow a single repository in a given context.
        if hasattr(self, '_v_repository'):
            return self._v_repository
        try:
            items = self.superValues('Repository')
        except BaseException:
            items = self.aq_inner.aq_parent.superValues('Repository')
        result = items and items[0] or None
        if result is None:
            raise VersionControlError(
                'No versioning repository was found.'
            )
        self._v_repository = result
        return result

    @security.public
    def isAVersionableResource(self, object):
        return self.getRepository().isAVersionableResource(self)

    @security.public
    def isUnderVersionControl(self):
        return hasattr(self, '__vc_info__')

    @security.public
    def isResourceUpToDate(self):
        return self.getRepository().isResourceUpToDate(self)

    @security.public
    def isResourceChanged(self):
        return self.getRepository().isResourceChanged(self)

    @security.public
    def getVersionInfo(self):
        return self.getRepository().getVersionInfo(self)

    @security.protected(use_vc_permission)
    def applyVersionControl(self, REQUEST=None):
        """Place a resource under version control."""
        repository = self.getRepository()
        object = repository.applyVersionControl(self)
        if REQUEST is not None:
            message = "The resource has been placed under version control."
            return object.versionControlMain(
                object, REQUEST,
                manage_tabs_message=message
            )

    @security.protected(use_vc_permission)
    def checkoutResource(self, REQUEST=None):
        """Checkout a version-controlled resource."""
        repository = self.getRepository()
        object = repository.checkoutResource(self)
        if REQUEST is not None:
            message = "The resource has been checked out."
            return object.versionControlMain(
                object, REQUEST,
                manage_tabs_message=message
            )

    @security.protected(use_vc_permission)
    def checkinResource(self, message='', REQUEST=None):
        """Checkout a version-controlled resource."""
        repository = self.getRepository()
        object = repository.checkinResource(self, message)
        version = object.getVersionInfo().version_id
        if REQUEST is not None:
            message = (
                "The resource has been checked in [version %s]." % version)
            return object.versionControlMain(
                object, REQUEST,
                manage_tabs_message=message
            )

    @security.protected(use_vc_permission)
    def uncheckoutResource(self, REQUEST=None):
        """Uncheckout a version-controlled resource."""
        repository = self.getRepository()
        object = repository.uncheckoutResource(self)
        version = object.getVersionInfo().version_id
        if REQUEST is not None:
            message = "The resource has been reverted to version %s." % version
            return object.versionControlMain(
                object, REQUEST,
                manage_tabs_message=message
            )

    @security.protected(use_vc_permission)
    def updateResource(self, selector, REQUEST=None):
        """Update a version-controlled resource."""
        repository = self.getRepository()
        if selector == 'LATEST_VERSION':
            selector = None
        object = repository.updateResource(self, selector)
        version = object.getVersionInfo().version_id
        if REQUEST is not None:
            message = "The resource has been updated to version %s." % version
            return object.versionControlMain(
                object, REQUEST,
                manage_tabs_message=message
            )

    @security.protected(use_vc_permission)
    def labelResource(self, label, force=0, REQUEST=None):
        """Label a version-controlled resource."""
        repository = self.getRepository()
        object = repository.labelResource(self, label, force)
        if REQUEST is not None:
            message = "The label has been applied to this resource."
            return object.versionControlMain(
                object, REQUEST,
                manage_tabs_message=message
            )

    @security.protected(use_vc_permission)
    def getVersionIds(self):
        return self.getRepository().getVersionIds(self)

    @security.protected(use_vc_permission)
    def getLabelsForHistory(self):
        return self.getRepository().getLabelsForHistory(self)

    @security.protected(use_vc_permission)
    def getLabelsForVersion(self):
        return self.getRepository().getLabelsForVersion(self)

    @security.protected(use_vc_permission)
    def getLogEntries(self):
        return self.getRepository().getLogEntries(self)


InitializeClass(VersionSupport)



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
"""Test the VersionHistory internals."""
import unittest

from .common import common_setUp
from .common import common_tearDown


class VersionHistoryTests(unittest.TestCase):

    setUp = common_setUp
    tearDown = common_tearDown

    def testBranchHaveName(self):
        self.repository.applyVersionControl(self.document1)
        info = self.repository.getVersionInfo(self.document1)
        history = self.repository.getVersionHistory(info.history_id)
        branch = history.createBranch('foo', None)
        self.assertEqual(branch.getId(), 'foo')
